@@ -4,15 +4,13 @@ use strict;
 use warnings;
 
 use Carp;
-use Digest::HMAC_SHA1;
-use HTTP::Date qw(time2str);
-use MIME::Base64 qw(encode_base64);
+use HTTP::Date ();
 use LWP::UserAgent;
 use URI::Escape qw(uri_escape_utf8);
+use Amazon::S3::Thin::Signer;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
-my $AMAZON_HEADER_PREFIX = 'x-amz-';
 my $METADATA_PREFIX      = 'x-amz-meta-';
 
 sub new {
@@ -96,7 +94,7 @@ sub copy_object {
 sub put_object {
     my ($self, $bucket, $key, $content, $headers) = @_;
     croak 'must specify key' unless $key && length $key;
-    
+
     if ($headers->{acl_short}) {
         $self->_validate_acl_short($headers->{acl_short});
         $headers->{'x-amz-acl'} = $headers->{acl_short};
@@ -151,6 +149,11 @@ sub _uri {
       : $bucket . "/";
 }
 
+sub _urlencode {
+    my ($self, $unencoded) = @_;
+    return uri_escape_utf8($unencoded, '^A-Za-z0-9_-');
+}
+
 sub _validate_acl_short {
     my ($self, $policy_name) = @_;
 
@@ -164,7 +167,7 @@ sub _validate_acl_short {
 # EU buckets must be accessed via their DNS name. This routine figures out if
 # a given bucket name can be safely used as a DNS name.
 sub _is_dns_bucket {
-    my $bucketname = $_[0];
+    my ($self, $bucketname) = @_;
 
     if (length $bucketname > 63) {
         return 0;
@@ -189,134 +192,44 @@ sub _compose_request {
     croak 'must specify path'   unless defined $path;
     $headers ||= {};
     $metadata ||= {};
-    my $http_headers = $self->_merge_meta($headers, $metadata);
 
-    $self->_add_auth_header($http_headers, $method, $path)
-      unless exists $headers->{Authorization};
+    # generates an HTTP::Headers objects given one hash that represents http
+    # headers to set and another hash that represents an object's metadata.
+    my $http_headers = HTTP::Headers->new;
+    while (my ($k, $v) = each %$headers) {
+        $http_headers->header($k => $v);
+    }
+    while (my ($k, $v) = each %$metadata) {
+        $http_headers->header("$METADATA_PREFIX$k" => $v);
+    }
+
+    # do we need check existance of Authorization ?
+    if (! exists $headers->{Authorization}) {
+        if (not $http_headers->header('Date')) {
+            $http_headers->header(Date => HTTP::Date::time2str(time));
+        }
+
+        my $signer = Amazon::S3::Thin::Signer->new($self->{aws_secret_access_key});
+        my $signature = $signer->calculate_signature($method, $path, $http_headers);
+        $http_headers->header(
+            Authorization => sprintf("AWS %s:%s"
+                                     , $self->{aws_access_key_id}
+                                     , $signature));
+    }
+
     my $protocol = $self->secure ? 'https' : 'http';
     my $host     = $self->host;
-    my $url      = "$protocol://$host/$path";
-    if ($path =~ m{^([^/?]+)(.*)} && _is_dns_bucket($1)) {
+    my $url;
+
+    if ($path =~ m{^([^/?]+)(.*)} && $self->_is_dns_bucket($1)) {
         $url = "$protocol://$1.$host$2";
+    } else {
+        $url = "$protocol://$host/$path";
     }
 
     return HTTP::Request->new($method, $url, $http_headers, $content);
 }
 
-sub _add_auth_header {
-    my ($self, $headers, $method, $path) = @_;
-
-    if (not $headers->header('Date')) {
-        $headers->header(Date => time2str(time));
-    }
-    my $canonical_string = $self->_canonical_string($method, $path, $headers);
-    my $encoded_canonical = $self->_encode($canonical_string);
-    $headers->header(
-        Authorization => sprintf("AWS %s:%s"
-                                 , $self->{aws_access_key_id}
-                                 , $encoded_canonical));
-}
-
-# generates an HTTP::Headers objects given one hash that represents http
-# headers to set and another hash that represents an object's metadata.
-sub _merge_meta {
-    my ($self, $headers, $metadata) = @_;
-    $headers  ||= {};
-    $metadata ||= {};
-
-    my $http_header = HTTP::Headers->new;
-    while (my ($k, $v) = each %$headers) {
-        $http_header->header($k => $v);
-    }
-    while (my ($k, $v) = each %$metadata) {
-        $http_header->header("$METADATA_PREFIX$k" => $v);
-    }
-
-    return $http_header;
-}
-
-# generate a canonical string for the given parameters.  expires is optional and is
-# only used by query string authentication.
-sub _canonical_string {
-    my ($self, $method, $path, $headers, $expires) = @_;
-    my %interesting_headers = ();
-    while (my ($key, $value) = each %$headers) {
-        my $lk = lc $key;
-        if (   $lk eq 'content-md5'
-            or $lk eq 'content-type'
-            or $lk eq 'date'
-            or $lk =~ /^$AMAZON_HEADER_PREFIX/)
-        {
-            $interesting_headers{$lk} = $self->_trim($value);
-        }
-    }
-
-    # these keys get empty strings if they don't exist
-    $interesting_headers{'content-type'} ||= '';
-    $interesting_headers{'content-md5'}  ||= '';
-
-    # just in case someone used this.  it's not necessary in this lib.
-    $interesting_headers{'date'} = ''
-      if $interesting_headers{'x-amz-date'};
-
-    # if you're using expires for query string auth, then it trumps date
-    # (and x-amz-date)
-    $interesting_headers{'date'} = $expires if $expires;
-
-    my $buf = "$method\n";
-    foreach my $key (sort keys %interesting_headers) {
-        if ($key =~ /^$AMAZON_HEADER_PREFIX/) {
-            $buf .= "$key:$interesting_headers{$key}\n";
-        }
-        else {
-            $buf .= "$interesting_headers{$key}\n";
-        }
-    }
-
-    # don't include anything after the first ? in the resource...
-    $path =~ /^([^?]*)/;
-    $buf .= "/$1";
-
-    # ...unless there is an acl or torrent parameter
-    if ($path =~ /[&?]acl($|=|&)/) {
-        $buf .= '?acl';
-    }
-    elsif ($path =~ /[&?]torrent($|=|&)/) {
-        $buf .= '?torrent';
-    }
-    elsif ($path =~ /[&?]location($|=|&)/) {
-        $buf .= '?location';
-    }
-
-    return $buf;
-}
-
-sub _trim {
-    my ($self, $value) = @_;
-    $value =~ s/^\s+//;
-    $value =~ s/\s+$//;
-    return $value;
-}
-
-# finds the hmac-sha1 hash of the canonical string and the aws secret access key and then
-# base64 encodes the result (optionally urlencoding after that).
-sub _encode {
-    my ($self, $str, $urlencode) = @_;
-    my $hmac = Digest::HMAC_SHA1->new($self->{aws_secret_access_key});
-    $hmac->add($str);
-    my $b64 = encode_base64($hmac->digest, '');
-    if ($urlencode) {
-        return $self->_urlencode($b64);
-    }
-    else {
-        return $b64;
-    }
-}
-
-sub _urlencode {
-    my ($self, $unencoded) = @_;
-    return uri_escape_utf8($unencoded, '^A-Za-z0-9_-');
-}
 
 1;
 
@@ -398,11 +311,9 @@ That's why I made this module.
 
 lots of APIs are not implemented yet.
 
-=head1 SUPPORT
+=head1 REPOSITORY
 
-Bugs should be reported via Github
-
-https://github.com/DQNEO/Amazon-S3-Thin/issues
+https://github.com/DQNEO/Amazon-S3-Thin
 
 =head1 LICENSE
 
@@ -418,15 +329,16 @@ DQNEO
 =head2 ORIGINAL AUTHOR
 
 Timothy Appnel <tima@cpan.org> L<Amazon::S3>
+https://github.com/tima/perl-amazon-s3
 
 =head1 SEE ALSO
 
 L<Amazon::S3>, L<Net::Amazon::S3>
 
-API Reference : REST API
+Amazon S3 API Reference : REST API
 http://docs.aws.amazon.com/AmazonS3/latest/API/APIRest.html
 
-API Reference : List of Error Codes
+Amazon S3 API Reference : List of Error Codes
 http://docs.aws.amazon.com/AmazonS3/latest/API/ErrorResponses.html#ErrorCodeList
 
 =cut
